/*
 * $Id: readdir.c,v 1.1 2001/09/05 19:10:01 cwolf Exp $
 * $Name:  $
 *
 * Portions copyright Apache Software Foundation (see below)
 * 
 * Functions defined are opendir(), readdir() and closedir() with the 
 * same prototypes as the normal dirent.h implementation. 
 * 
 * Does not implement telldir(), seekdir(), rewinddir() or scandir(). 
 * The dirent struct is compatible with Unix, except that d_ino is 
 * always 1 and d_off is made up as we go along. 
 * 
 * The DIR typedef is not compatible with Unix. 
 **********************************************************************/ 
#include <malloc.h> 
#include <string.h> 
#include <errno.h> 

#include "dirent.h" 


DIR *opendir(char *dir) 
{ 
    DIR *dp; 
    char *filespec; 
    long handle; 
    int index; 


    filespec = malloc(strlen(dir) + 2 + 1); 
    strcpy(filespec, dir); 
    index = strlen(filespec) - 1; 
    if (index >= 0 && (filespec[index] == '/' || filespec[index] == '\\')) 
        filespec[index] = '\0'; 
    strcat(filespec, "/*"); 


    dp = (DIR *)malloc(sizeof(DIR)); 
    dp->offset = 0; 
    dp->finished = 0; 
    dp->dir = strdup(dir); 


    if ((handle = _findfirst(filespec, &(dp->fileinfo))) < 0) { 
        if (errno == ENOENT) 
            dp->finished = 1; 
        else 
        return NULL; 
    } 


    dp->handle = handle; 
    free(filespec); 


    return dp; 
} 


struct dirent *readdir(DIR *dp) 
{ 
    if (!dp || dp->finished) return NULL; 


    if (dp->offset != 0) { 
        if (_findnext(dp->handle, &(dp->fileinfo)) < 0) { 
            dp->finished = 1; 
            return NULL; 
        } 
    } 
    dp->offset++; 


    strncpy(dp->dent.d_name, dp->fileinfo.name, _MAX_FNAME); 
    dp->dent.d_ino = 1; 
    dp->dent.d_reclen = strlen(dp->dent.d_name); 
    dp->dent.d_off = dp->offset; 


    return &(dp->dent); 
} 


int closedir(DIR *dp) 
{ 
    if (!dp) return 0; 
    _findclose(dp->handle); 
    if (dp->dir) free(dp->dir); 
    if (dp) free(dp); 


    return 0; 
} 
/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * Portions of this software are based upon public domain software
 * originally written at the National Center for Supercomputing Applications,
 * University of Illinois, Urbana-Champaign.
 */
